/*
* Copyright (C) fastman92 <fastman92@gmail.com>, website: http://fastman92.com
* Licensed under the MIT License, see LICENSE at top level directory.
*
*/

// dllmain.cpp : Defines the entry point for the DLL application.

#include "ForOtherProjects/Exports.h"
#include "Loader.h"
#include "Log.h"

#include "../fastman92 limit adjuster/Source files/ForOtherProjects/Common/common.h"

#include <fileIO.h>
#include <MultiPlatformSupport/PlatformGeneralisation.h>

#include <pthread.h>

#ifdef IS_PLATFORM_WIN
#include <windows.h>
BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}
#endif

#ifdef IS_PLATFORM_ANDROID
#include <jni.h>
#include <errno.h>

#include <string.h>
// #include <unistd.h>

#include <android/log.h>

#include <stdint.h>

#include <dirent.h>
#include <dlfcn.h>
#include <jni.h>
#include <link.h>
#include <stdio.h>
#include <limits.h>

bool strEndCompare(const char* str, const char* needle)
{
	int l_str = strlen(str);
	int l_needle = strlen(needle);

	if (l_needle > l_str)
		return false;

	return !memcmp(str + l_str - l_needle, needle, l_needle);
}

#ifdef IS_ARCHITECTURE_ARM32

// Replaces Jni_Onload
jint NAKED JNIEXPORT ProcedureToAddIntoFile(JavaVM *vm, void *reserved)
{
	__asm volatile(

	"push    {R4-R10, lr}\n"

		"mov r4, r0\n"
		"mov r5, r1\n"

		// ProcessPluginLoadingWithJava
		"ldr r2, appIdentifier%=\n"	// application identifier

		"ldr r3, 6f\n"	// base address
		"add r3, pc\n"

		"ldr r6, 4f\n"	// ProcessPluginLoading in .fastman92.got
		"add r6, pc\n"
		"ldr r6, [r6]\n"
		"blx r6\n"

		// r4 and r5 is used

		// pointer to .init_array
		"ldr r6, 1f\n"
		"add r6, pc\n"

		// number of elements
		"ldr r7, 2f\n"

		// loop
		"initArrayLoop:"

		"ldr r8, [r6]\n"

		"cmp r8, #0\n"	// check if pointer to function is not zero
		"beq goToNextElement\n"

		"cmp r8, #-1\n"	// check if pointer to function is not -1
		"beq goToNextElement\n"

		"blx r8\n"

		"goToNextElement:\n"

		"add r6, #4\n"
		"sub r7, #1\n"
		"cmp r7, #0\n"
		"bne initArrayLoop\n"

		// call real JNI_OnLoad
		"mov r0, r4\n"
		"mov r1, r5\n"

		"ldr r6, 3f\n"
		"add r6, pc\n"
		"blx r6\n"

		// return
		"pop     {R4-R10, pc}\n"

		".align 4\n"
		"appIdentifier%=:.word 0x874327\n" // application identifier, offset: -24
		"6:.word 0x4167463\n" // base address, offset: -20
		"1:.word 0x7311313\n" // pointer to .init_array, offset: -16
		"2:.word 0x12345678\n" // number of elements in the .init_array, offset: -12
		"3:.word 0x6322525\n" // JNI on load, offset: -8
		"4:.word 0x25735353\n" // ProcessPluginLoadingWithJava, offset: -4
		: :
		);
}
#endif

extern "C"
{
	CPluginLoader g_Loader;
	static char libPluginPath[PATH_MAX] = "";

	#if 0
	// This function will load the SO libraries
	void JNIEXPORT ProcessPluginLoading()
	{
		LOGI("Starting plugin loader, shared objects get loaded at the end of processing .init_array in the host library.");
		DoLoading();
	}
	#endif

	#if 0
	void JNIEXPORT ProcessSAMPloading(JavaVM* javaVm, void* reserved, unsigned int applicationIdentifier, void* baseAddressForApplicationLibrary)
	{
		g_Loader.GetApplicationLibDirectoryPath(libPluginPath);
		char path[PATH_MAX];
		sprintf(path, "%s/libsamp.so", libPluginPath);

		void* pluginHandle = dlopen(path, RTLD_GLOBAL | RTLD_LAZY);

		type_JNI_OnLoad pJni_OnLoad = (type_JNI_OnLoad)dlsym(pluginHandle, "JNI_OnLoad");

		if (pJni_OnLoad)
			pJni_OnLoad(javaVm, reserved);
	}
	#endif

	#if 1
	// This function will load the SO libraries
	void JNIEXPORT ProcessPluginLoadingWithJava(JavaVM* javaVm, void* reserved, unsigned int applicationIdentifier, void* baseAddressForApplicationLibrary)
	{
		OutputFormattedDebugString("Starting plugin loader, shared objects get loaded, when JNI_OnLoad begins to execute in the host library.");
		
		if (!g_Loader.InitialiseLoading(javaVm, reserved, (const char*)&applicationIdentifier, baseAddressForApplicationLibrary))
		{
			OutputFormattedDebugString("Failed to initialise.");
			return;
		}

		g_Loader.GetApplicationLibDirectoryPath(libPluginPath);
		
		g_Loader.DoPluginLoading(libPluginPath);
	}
	#endif
}

// Exported functions
extern "C"
{
	// Compares a library identifier to specified value, for example against "SA20"
	// Identifiers take 4 bytes. That's a FourCC ("four-character code").
	F92_LA_API bool TestLibIdentifier(const char* pIdentifier, const char* valueToTestAgainst)
	{
		return pIdentifier[0] == valueToTestAgainst[0]
			&& pIdentifier[1] == valueToTestAgainst[1]
			&& pIdentifier[2] == valueToTestAgainst[2]
			&& pIdentifier[3] == valueToTestAgainst[3];
	}

	// Key to access JNIEnv* for current thread
	static pthread_key_t CurrentThreadJNIenvKey = 0;

	// Attaches the thread to Java machine and returns JNIEnv* pointer
	F92_LA_API JNIEnv* GetCurrentThreadJNIEnv()
	{
		JNIEnv* pJNIenv;

		if (CurrentThreadJNIenvKey)
		{
			pJNIenv = (JNIEnv*)pthread_getspecific(CurrentThreadJNIenvKey);

			if (pJNIenv)
				return pJNIenv;
		}
		else
			pthread_key_create(&CurrentThreadJNIenvKey, 0);

		// Need to attach the thread
		if (g_Loader.m_javaVm->AttachCurrentThread(&pJNIenv, NULL) == JNI_OK)
			pthread_setspecific(CurrentThreadJNIenvKey, pJNIenv);

		return pJNIenv;
	}

	// Sets the path to external storage directory
	F92_LA_API const char* GetApplicationPackageName()
	{
		return g_Loader.ms_packageName;
	}

	// Returns the pointer to initial directory
	F92_LA_API const char* GetInitialDirectoryPath()
	{
		return g_Loader.ms_InitialDirectory;
	}

	// Returns the pointer to root directory
	F92_LA_API const char* GetStorageRootDirectoryPath()
	{
		return g_Loader.ms_StorageRootDirectory;
	}

	// Returns the pointer to storage directory
	F92_LA_API const char* GetStorageRootBaseDirectoryPath()
	{
		return g_Loader.ms_StorageRootBaseDirectory;
	}

	// Returns the pointer to cache directory
	F92_LA_API const char* GetCacheDirectoryPath()
	{
		return g_Loader.ms_CacheDirectory;
	}

	// Returns jobject of main activity during launch
	F92_LA_API jobject GetMainActivityDuringLaunch()
	{
		return g_Loader.ms_mainActivityDuringLaunch;
	}
}
#endif